﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using AddinExpress.MSO;
using System.Collections.Generic;
using System.IO;
using System.Reflection;

namespace NewMailUnleashedPart4
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("54A222C0-043C-46E4-8DEB-25FA49A4CE60"), ProgId("NewMailUnleashedPart4.AddinModule")]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }

        private ADXOutlookAppEvents adxOutlookEvents;
        private DateTime LastReceivedDate = DateTime.Now;
        private Timer mailCheckTimer;

        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;

        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.adxOutlookEvents = new AddinExpress.MSO.ADXOutlookAppEvents(this.components);
            this.mailCheckTimer = new System.Windows.Forms.Timer(this.components);
            // 
            // adxOutlookEvents
            // 
            this.adxOutlookEvents.NewMailEx += new AddinExpress.MSO.ADXOlNewMailEx_EventHandler(this.adxOutlookEvents_NewMailEx);
            this.adxOutlookEvents.SyncEnd += new AddinExpress.MSO.ADXOlSyncObjectEnd_EventHandler(this.adxOutlookEvents_SyncEnd);
            // 
            // mailCheckTimer
            // 
            this.mailCheckTimer.Enabled = true;
            this.mailCheckTimer.Interval = 1000;
            this.mailCheckTimer.Tick += new System.EventHandler(this.mailCheckTimer_Tick);
            // 
            // AddinModule
            // 
            this.AddinName = "NewMailUnleashedPart4";
            this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
            this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);
            this.OnSendMessage += new AddinExpress.MSO.ADXSendMessage_EventHandler(this.AddinModule_OnSendMessage);

        }
        #endregion

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private SyncItems syncItems = new SyncItems();

        private void AddinModule_AddinStartupComplete(object sender, EventArgs e)
        {
            syncItems.NewMailReceived += syncItems_NewMailReceived;

            if (!File.Exists(syncItems.FileName))
            {
                syncItems.InitializeSyncTimesAccounts(DateTime.Now, this.OutlookApp);
                syncItems.Save();
            }
            else
            {
                syncItems.Load();
            }
            mailCheckTimer.Start();
        }

        void syncItems_NewMailReceived(object sender, SyncItem e)
        {
            // Use this event to handle all incoming e-mails
            System.Diagnostics.Debug.WriteLine(
                String.Format("{0} ->{1}{2}", e.ReceivedTime, e.EntryID, Environment.NewLine));
        }

        private void adxOutlookEvents_SyncEnd(object sender, object syncObject)
        {
            mailCheckTimer.Start();
        }

        private void LoopThroughAccountFolders()
        {
            Outlook.NameSpace nameSpace = OutlookApp.GetNamespace("MAPI");
            Outlook.Folders accountFolders = nameSpace.Folders;
            for (int i = 1; i <= accountFolders.Count; i++)
            {
                Outlook.MAPIFolder accountFolder = accountFolders.Item(i);
                SyncTime syncTime = syncItems.SyncTimes.Find(s => s.AccountID == accountFolder.EntryID);

                if (syncTime != null)
                {
                    LastReceivedDate = DateTime.MinValue;
                    ScanFolder(accountFolder, syncTime.LatestTime, accountFolder.EntryID);
                }

                if (LastReceivedDate > DateTime.MinValue)
                {
                    syncItems.SetSyncTime(LastReceivedDate, accountFolder.EntryID);
                    syncItems.Save();
                }

                if (accountFolder != null)
                    Marshal.ReleaseComObject(accountFolder);
            }
            if (accountFolders != null)
                Marshal.ReleaseComObject(accountFolders);
            if (nameSpace != null)
                Marshal.ReleaseComObject(nameSpace);
        }

        private void ScanFolder(Outlook.MAPIFolder folder, DateTime receivedTime, string accountId)
        {
            if (folder.DefaultItemType == Outlook.OlItemType.olMailItem)
            {
                Outlook.Items folderItems = folder.Items;
                Outlook.Items filteredItems = folderItems.Restrict(String.Format("[ReceivedTime] >= '{0}'", receivedTime.AddMinutes(-1).ToString("g")));
                Marshal.ReleaseComObject(folderItems);

                for (int i = 1; i <= filteredItems.Count; i++)
                {
                    object outlookItem = filteredItems.Item(i);

                    if (!IsSentItem(outlookItem))
                    {
                        if (outlookItem is Outlook.MailItem)
                        {
                            Outlook.MailItem outlookMail = (Outlook.MailItem)outlookItem;

                            SyncItem syncItem = new SyncItem();
                            syncItem.EntryID = outlookMail.EntryID;
                            syncItem.SenderSubject = outlookMail.Subject;
                            syncItem.ReceivedTime = outlookMail.ReceivedTime;
                            syncItem.AccountID = accountId;
                            syncItems.Add(syncItem);

                            if (syncItem.ReceivedTime > LastReceivedDate)
                                LastReceivedDate = syncItem.ReceivedTime;
                        }
                    }
                    Marshal.ReleaseComObject(outlookItem);
                }
            }

            Outlook.Folders subFolders = folder.Folders;
            for (int i = 1; i <= subFolders.Count; i++)
            {
                Outlook.MAPIFolder folderToScan = subFolders.Item(i);
                ScanFolder(folderToScan, receivedTime, accountId);
                Marshal.ReleaseComObject(folderToScan);
            }
            Marshal.ReleaseComObject(subFolders);
        }

        private void AddinModule_OnSendMessage(object sender, ADXSendMessageEventArgs e)
        {
            if (e.Message == 0x04001)
            {
                mailCheckTimer.Stop();
                LoopThroughAccountFolders();
            }
        }

        private void mailCheckTimer_Tick(object sender, EventArgs e)
        {
            this.SendMessage(0x04001, IntPtr.Zero, IntPtr.Zero);
        }

        private void adxOutlookEvents_NewMailEx(object sender, string entryIDCollection)
        {
            Outlook.NameSpace nameSpace = null;
            object outlookItem = null;

            try
            {
                nameSpace = OutlookApp.GetNamespace("MAPI");

                string[] entryIds = entryIDCollection.Split(',');
                for (int i = 0; i < entryIds.Length; i++)
                {
                    outlookItem = nameSpace.GetItemFromID(entryIds[i]);

                    if (outlookItem != null)
                    {
                        if (outlookItem is Outlook.MailItem)
                        {
                            Outlook.MailItem outlookMail = (Outlook.MailItem)outlookItem;

                            SyncItem syncItem = new SyncItem();
                            syncItem.EntryID = outlookMail.EntryID;
                            syncItem.SenderSubject = outlookMail.Subject;
                            syncItem.ReceivedTime = outlookMail.ReceivedTime;
                            syncItem.AccountID = GetItemAccountID(outlookItem);
                            syncItems.Add(syncItem);
                            syncItems.Save();
                        }

                        Marshal.ReleaseComObject(outlookItem);
                    }
                }
            }
            finally
            {
                if (nameSpace != null)
                    Marshal.ReleaseComObject(nameSpace);
            }
        }

        private bool IsNamespace(object Folder)
        {
            if (Folder is Outlook.NameSpace)
            {
                return true;
            }
            return false;
        }

        private string GetItemAccountID(object item)
        {
            string accountID = string.Empty;
            object parent = null;

            try
            {
                parent = item.GetType().InvokeMember("Parent",
                    BindingFlags.GetProperty, null, item, new object[] { });
                if (IsNamespace(parent))
                {
                    object entryID = item.GetType().InvokeMember("EntryID",
                        BindingFlags.GetProperty, null, item, new object[] { }); ;
                    if (entryID != null)
                    {
                        accountID = entryID.ToString();
                    }
                }
                else
                {
                    accountID = GetItemAccountID(parent);
                }
            }
            finally
            {
                if (parent != null)
                    Marshal.ReleaseComObject(parent);
            }
            return accountID;
        }

        private bool IsSentItem(object item)
        {
            bool sent = false;
            string receivedByName = string.Empty;

            if (item is Outlook.MailItem)
            {
                Outlook.MailItem receivedMail = (Outlook.MailItem)item;
                sent = receivedMail.Sent;
                receivedByName = receivedMail.ReceivedByName;
            }

            if ((!sent) || (sent && (((receivedByName != null) &&
                ((receivedByName as string) == string.Empty)) || (receivedByName == null))))
            {
                return true;
            }
            return false;
        }
    }
}
